<?php

/**
 * WP_CF7DB_Admin_Menu class
 *
 * @author   Devendra Mer <davanwp@gmail.com>
 * @package  WP_CF7DB
 * @since    1.0.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WP_CF7DB_Admin_Menu
{
    
    /**
     * Plugin Constructor method
    */
    public function __construct()
    {
        add_action( 'admin_menu', array($this, 'cf7db_menu_page' ) );
    }

     /**
     * Menu item will allow us to load the page to display the table
     */
    public function cf7db_menu_page()
    {

        add_menu_page( 
            __( 'CF7 DB', 'wp-cf7db' ),
            __( 'CF7 DB', 'wp-cf7db' ),
            'manage_options',
            WP_CF7DB_SLUG,
            array($this, 'cf7db_form_list'),
            'dashicons-database'
        );

    }

    /**
     * Display the list table page
     *
     * @return Void
     */
    public function cf7db_form_list()
    {
        if ( ! class_exists('WPCF7_ContactForm') ) {

           wp_die( 'You Must need to activate <a href="https://wordpress.org/plugins/contact-form-7/" target="_blank">Contact form 7</a> plugin.' );
        }

        $form_id  = empty($_GET['form_id']) ? 0 : (int) esc_attr($_GET['form_id']);
        $record_id = empty($_GET['record_id']) ? 0 : (int) esc_attr($_GET['record_id']);

        if ( !empty($form_id) && empty($_GET['record_id']) ) {
            
            new WP_CF7DB_Admin_Records();
            return;
        }

        if( !empty($form_id) && !empty($record_id) ){

            new WP_CF7DB_Record_Details();
            return;
        }
       
        $ListTable = new WP_CF7DB_Main_List_Table();
        $ListTable->prepare_items();
        ?>
            <div class="wrap">
                <h2><?php _e( 'Forms List', 'wp-cf7db' ); ?></h2>
                <?php $ListTable->display(); ?>
            </div>
        <?php
    }

}

// WP_List_Table is not loaded automatically so we need to load it in our application
if( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}
/**
 * Create a new table class that will extend the WP_List_Table
 */
class WP_CF7DB_Main_List_Table extends WP_List_Table
{

    /**
     * Prepare the items for the table to process
     *
     * @return Void
     */
    public function prepare_items()
    {

        global $wpdb;
        $wp_cf7db    = apply_filters( 'wp_cf7db_database', $wpdb );
        $table_name  = $wp_cf7db->prefix.'cf7db_forms';
        $columns     = $this->get_columns();
        $hidden      = $this->get_hidden_columns();
        $data        = $this->table_data();
        $perPage     = 10;
        $currentPage = $this->get_pagenum();
        $count_forms = wp_count_posts('wpcf7_contact_form');
        $totalItems  = $count_forms->publish;


        $this->set_pagination_args( array(
            'total_items' => $totalItems,
            'per_page'    => $perPage
        ) );

        $this->_column_headers = array($columns, $hidden );
        $this->items = $data;
    }
    
    /**
     * Override the parent columns method. Defines the columns to use in your listing table
     *
     * @return Array
     */
    public function get_columns()
    {


        $columns = array(
            'name' => __( 'Form Name', 'wp-cf7db' ),
            'count'=> __( 'Total Submission ', 'wp-cf7db' )
        );

        return $columns;
    }
    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Get the table data
     *
     * @return Array
     */
    private function table_data()
    {
        global $wpdb;

        $wp_cf7db    = apply_filters( 'wp_cf7db_database', $wpdb );
        $table_name  = $wp_cf7db->prefix.'cf7db_forms';
        $data         = array();
        $page         = $this->get_pagenum();
        $page         = $page - 1;
        $start        = $page * 10;

        $args = array(
            'post_type'=> 'wpcf7_contact_form',
            'order'    => 'ASC',
            'posts_per_page' => 10,
            'offset' => $start
        );

        $the_query = new WP_Query( $args );

        while ( $the_query->have_posts() ) : $the_query->the_post();
            $form_id = get_the_id();
            $totalItems   = $wp_cf7db->get_var("SELECT COUNT(*) FROM $table_name WHERE form_id = $form_id");
            $title = get_the_title();
            $link  = "<a class='row-title' href=admin.php?page=".WP_CF7DB_SLUG."&form_id=$form_id>%s</a>";
            $data_value['name']  = sprintf( $link, $title );
            $data_value['count'] = sprintf( $link, $totalItems );
            $data[] = $data_value;
        endwhile;

        return $data;
    }
    /**
     * Define what data to show on each column of the table
     *
     * @param  Array $item        Data
     * @param  String $column_name - Current column name
     *
     * @return Mixed
     */
    public function column_default( $item, $column_name )
    {
        return $item[ $column_name ];

    }

}